//---------------------------------------------------------------------------
// JrDebug demo that verifies that all the macros can be compiled/parsed without error
// It basically attempts to use all functions, etc.
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Include the Debugging Library files
//  Note that only our main .cpp needs to include the jrdebug_main.cpp file
//  alternatively, you could add this cpp to your project or makefile.
#include "../../jrdebug_main.cpp"
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#include <string>
#include <iostream>
#include <cassert>
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// create a dummy func we use later
void dummyfunc();
void dummyfunc() { debugout << "test message"; };
// register it, as a test
JrDebugRegisterTestFunction ("dummytest","a dummy test",dummyfunc,"")
// another forward declaration for helper
void TestNestedIfs(bool a, bool b);
void TestNestedIfs2(int i);
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Helper callback object - can be used if user code want to intercept events
//  To use, derive your own version of this class, create an instance of it, and pass it to the static
//  function JrDebug::SetJrDebugCallbackPointer(class JrDebugCallback *p);
// Callback should return true to process as normal, or false to not send output as normal
class MyDebugCatcher : public JrDebugCallback {
public:
	bool EventNotify(const char *fulltext) { std::cout << "Local debug catcher got: "<<fulltext;return true;};
};
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
int main(int argc, char *argv[])
{
	std::string dummystring="test string";

	// automatically parse commandline options and exit if appropriate
	if (JrDebugParseCommandlineArgc("App Decription",&argc,argv,"-db"))
		return 0;
	if (JrDebugParseCommandlineStr("App Decription","","-db"))
		return 0;
	if (JrDebugParseCommandline("App Decription",&argc,argv,"","","-db","-dbo text;stdout"))
		return 0;

	if (JrDebugParseCommandlineStr("SystemTrayTrigger","","-db -dbo text;stdout"))
		return 0;

	// a test of debug streams and printfs as single line consequences of an if statement
	// this is due to need to have nested if statements and the limits of macro expansion; as an example:
	// this is made to be a syntax error and wont compile:
	TestNestedIfs(true,true);
	TestNestedIfs(false,true);
	TestNestedIfs(false,false);
	TestNestedIfs2(30);
	TestNestedIfs2(20);

	// enabled flag - we use this check before all print functions to minimize runtime costs when compiled in but disabled
	if (JrDebugEnabled)
		dbprintf("jrd is enabled.");
	JrDebugDisable("reason why it's being disabled");
	JrDebugEnable("reason why it's being enabled");

	// temporarily disable and call
	JrDebugQuietCall(dummyfunc());

	// macros that allow us to quickly disable with very little cost, even when debugging is compiled in
	JrDebugPrintf("Here are some numbers %d,%d,%d",1,2,3);
	JrDebugPrintString(dummystring);
	JrDebugPrintfError("some numbers in an error: %d,%d",4,5);

	// ifdebug
	JrDebugIf(true) dbprintf("line one.");
	JrDebugIf(false) dbprintf("line two.");

	// replacement throw (triggers an log message of JrdException type, with severity set by static JrDebug::exceptionlevel; can be disabled by setting to JrDebugIgnoreLevel or calling JrDebug::DisableExceptionLogging())
	try
		{
		// assert and verify
		JrDebugAssert(2>1);
		JrDebugVerify(2<3);
		JrDebugThrow("test","this is a test throw");
		}
	catch (...)
		{
		}

	JrDebugStream << "ATTN:" << __FUNCTION__ ;

	// stream versions
	JrDebugStream << "some numbers: "<< 1 << "," << 2;
	JrDebugStreamError << "some err numbers: "<< 3 << "," << 4;

	// debug block
		{
		JrDebugBlock("testblock");
		JrDebugActivityPush("subloop");
		JrDebugBlockReport("here i am");
		JrDebugActivityPop("subloop");
		}

	// put debug block around a function call (note: func gets execute whether debugging is enabled or not)
	JrDebugEnclose(dummyfunc());

	// in case you want to test and confirm that debugging is compiled in
	// JrDebugErrorIfNotCompiledIn
	// JrDebugErrorIfCompiledIn

	// unit testing
	JrDebugRunTestFunction("manualtest","manually running dummy func test",dummyfunc);
	JrDebugTestRun(dummyfunc);
	JrDebugTest(1<2)
	JrDebugTestEx(2<3)
	JrDebugTestInfo("running a testcase",3<4)
	JrDebugTestExInfo("running another testcase",4<5)

	// callback func
	MyDebugCatcher mycatcher;
	JrDebugSetCallbackPointer(&mycatcher);

	// misc
	JrDebugEnableExceptionLogging(10);
	JrDebugSetDisplayPassingAsserts(true);

	// announce
	JrDebugAnnounce("Test Application",true,argc,argv);

	// accessors
	JrDebugSetDefaultOutputOptions("-dbo text;output.txt");

	// announce
	std::cout <<  "JrD Syntax Checking Program - this program does nothing useful.\n";

	// more display commands of dif types
	// alternative argument line with special parameters OutputDebugPrintf(MessageType,SeverityNumber,...)
	//void operator()(const JrdType inmtype , const jrd_stdstringT str);
	//void operator()(const JrdType inmtype , const char* format, ... );
	//void operator()(const JrdType inmtype , const int inseverity, const jrd_stdstringT str);
	//void operator()(const JrdType inmtype , const int inseverity, const char* format, ... );
	// alternative argument line with special parameters OutputDebugPrintf(MessageType,SubTypeString,ActivityString,SeverityNumber,...)
	//void operator()(const JrdType inmtype , const char *dmsubtypestr, const char *inactivity, const int inseverity, const jrd_stdstringT str);
	//void operator()(const JrdType inmtype , const char *dmsubtypestr, const char *inactivity, const int inseverity, const char* format, ... );
	dbprintf(JrdWarning,"test");
	dbprintf(JrdWarning,90,"test score %d",90);
	dbprintf(JrdWarning,"mywarntype","testing",80,"test");
	dbprintf(JrdWarning,dummystring);
	dbprintf(JrdWarning,90,dummystring);
	dbprintf(JrdWarning,"mywarntype","testing",80,dummystring);

	// alt streams
	//JrDebugs& operator()(const JrdType inmtype , const int inseverity=JrDebugDefaultLevel);
	//JrDebugs& operator()(const JrdType inmtype , const char *indmsubtypestr, const char *inactivity="", const int inseverity=JrDebugDefaultLevel);
	debugout(JrdWarning) << "stest1";
	debugout(JrdWarning,60) << "stest2";
	debugout(JrdWarning,"mywarntype","testing",50) << "stest3";

	// all done
	return 0;
}
//---------------------------------------------------------------------------



void TestNestedIfs(bool a, bool b)
{
	// announce
	debugout << "testing with a=" << a << " b="<<b;

	// stream style
	if (a)
		debugout << "a is true";
	else if (b)
		debugout << "a is false and b is true";
	else
		debugout << "a and b are both false";
	// printf style
	if (a)
		dbprintf("a is true");
	else if (b)
		dbprintf("a is false and b is true");
	else
		dbprintf("a and b are both false");
	// the above maps to something like
	if (a)
		if (false) ; else std::cout<<"a is true"<<std::endl;
	else if (b)
		if (false) ; else std::cout<<"a is false and b is true"<<std::endl;
	else
		if (false) ; else std::cout<<"a and b are both false"<<std::endl;
}


void TestNestedIfs2(int i)
{
	std::cout << std::endl;

	std::cout << i << ": ";
	if (i == 10) std::cout << 'A'<<std::endl;
	else if (i == 20) std::cout << 'B';
	else if (i == 30) std::cout << 'C'<<std::endl;
	else std::cout << 'D'<<std::endl;

	std::cout << "\n"<<std::endl;

	std::cout << i << ": ";
	if (i == 10) std::cout << 'A'<<std::endl;
	else if (i == 20) debugout << 'B';
	else if (i == 30) std::cout << 'C'<<std::endl;
	else std::cout << 'D'<<std::endl;

	std::cout << "\n"<<std::endl;
	
	std::cout << i << ": ";
	if (i == 10) debugout << 'A';
	else if (i == 20) debugout << 'B';
	else if (i == 30) debugout << 'C';
	else debugout << 'D';
	
	std::cout << "\n"<<std::endl;
}

